Package: ggraptR
Type: Package
Title: Allows Interactive Visualization of Data Through a Web Browser
        GUI
Version: 1.3
Authors@R: 
    c(person("Eugene", "Dubossarsky", role=c("aut", "cph", "cre"),
             email="eugene@presciient.com"),
	     person("Julius", "Neggo", role="ctb", comment="Lead Developer"),
	     person("Howard", "Song", role="aut", comment="Developer"),
	     person("Susan", "Wang", role="ctb", comment="Project Manager"),
	     person("Jason", "Widjaja", role="ctb", comment="Vignette"),
	     person("Jose", "Magana", role="ctb", comment="Package Manager and Developer"),
	     person("Sergey", "Skripko", role="ctb", comment="Developer"))
Description: Intended for both technical and non-technical users to create
    interactive data visualizations through a web browser GUI without writing any
    code.
License: GPL (>= 2)
Suggests: knitr, markdown
Depends: dplyr (>= 0.7.5), ggplot2 (>= 2.2.0), pacman (>= 0.4.6),
        purrr(>= 0.2.4), shiny (>= 0.12.2)
BugReports: https://github.com/cargomoose/raptR/issues
VignetteBuilder: knitr
RoxygenNote: 6.0.1
NeedsCompilation: no
Packaged: 2020-06-24 01:11:17 UTC; juliux
Author: Eugene Dubossarsky [aut, cph, cre],
  Julius Neggo [ctb] (Lead Developer),
  Howard Song [aut] (Developer),
  Susan Wang [ctb] (Project Manager),
  Jason Widjaja [ctb] (Vignette),
  Jose Magana [ctb] (Package Manager and Developer),
  Sergey Skripko [ctb] (Developer)
Maintainer: Eugene Dubossarsky <eugene@presciient.com>
Repository: CRAN
Date/Publication: 2020-06-24 05:00:03 UTC
Built: R 4.6.0; ; 2026-04-29 03:13:59 UTC; unix
