# ggraptR
[![](http://cranlogs.r-pkg.org/badges/ggraptR)](https://CRAN.R-project.org/package=ggraptR)

Rapid and Pretty Things in R : A shiny graphical user interface for your favourite ggplot graphics in R

![demo](inst/ggraptR/www/demo.gif)

## Installation  
``` 
install.packages('ggraptR', dependencies=TRUE)
```
